%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MatLab script that demonstrates generating tones with an AD9081 using
% an Analog Devices ADS9 pattern generator. The AD9xxx is configured
% using ACE UI.

% NOTE: 
%   1. The ACE application UI is used to configure the AD9xxx.
%   2. No external clock required for this demo. Use on board 122.88MHz
%
% 12/21/2021
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%configuration parameters
resolution = 16;
dataRate = 1500e6;
fdac = 12e9;
samples = 262144;
output_level = -8; %dBfs
dacclock = 12e9;
adcclock = dacclock/3;

%Instaniate DPGLite
dpg=NDPG9xxx();

% AD9xxx(IPC_Port, Subsystem_#, DAC_CLOCK, ADC_CLOCK)
mxfe = AD9xxx(2357,1,dacclock,adcclock);

%check if link is up
if(~dpg.linkup())
disp('No Link up is detected. Try to configure the ACE plugin first');
end

%generate IQ tone vectors (in my setup, I have four DACs. your number of dacs
%can be different. Number of I and Q data vectors must match JESD M
%parameter)
[Idata0,Qdata0,~, ~] = CreateTwoTone(110e6,100e6,samples, dataRate,output_level, resolution);
[Idata2,Qdata2,~] = CreateTone(1e6,samples, dataRate,output_level, resolution);
[Idata3,Qdata3,~] = CreateTone(1e6,samples, dataRate,output_level, resolution);

%loading external vector 
Idata1 = (importdata('test_vector_12MHz_I.txt'))';
Qdata1 = (importdata('test_vector_12MHz_Q.txt'))';

%Place all the vectors in order into one array
dpgArray0 = [Idata0; Qdata0; Idata1; Qdata1;Idata2; Qdata2; Idata3; Qdata3;];

% dummy read. improves stability
mxfe.captureAllChannels();

%Stop Vector from playing
dpg.stop();

%load the new vector
dpg.dowloadWFM(dpgArray0,1);

%play the loaded vector
dpg.play();

%Capture data on adc
%data=  mxfe.captureAllChannels();

%setNCOfreq(Freq, 'nco3, nco2, nco1, nco0') 
%Freq  - NCO frequency
%mask  - Which NCOs to set their frequency.
%        '1111' or '11111111' will write to all ncos
%        '1010' will write to NCO1 and NCO3
%        '0101' will write to NCO0 and NCO2
%mxfe.ADCsetNCOfreq(500e6, '1111')

%8 digit mask
%mxfe.ADCsetFineNCOfreq(0, '11111111')

%8 digit mask
%mxfe.DACsetFineNCOfreq(0, '11111111')

%8 digit mask
%mxfe.DACsetDCOffsetGain(0, '11111111')

%8 digit mask
%mxfe.ADCFineDDCGain(0, '11111111')

%4 digit mask
%mxfe.DACFSCSetting(26000, '1111')

%ToF - 0 for False/Off, 1 for True/On
%4 digit mask
%mxfe.ADCCoarseDDCGain(0, '1111')

%Count has to be a number -> 2^#
%mxfe.SampleCount(65536)

mxfe.ExportFile()

%Capture, plot data
%figure
%plot(data(1:2000,2));
