%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EXAMPLE MXFE AUTOMATION:
%
%  THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
%  INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
%  PARTICULAR PURPOSE ARE DISCLAIMED.
%
%  IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
%  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
%  RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
%  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
%  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
%  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



classdef AD9xxx
    
    properties
        clientManager
        client
        IPCport
        chip_cont
        brd_cont
        fdac
        fadc
        ads9_cont
    end
    methods
        function obj = AD9xxx(IPCport,subsys,varargin)
            
            obj.IPCport=IPCport;
            if ~exist('ClientManager','class')
                NET.addAssembly('C:\Program Files (x86)\Analog Devices\ACE\Client\AnalogDevices.Csa.Remoting.Clients.dll')
                import AnalogDevices.Csa.Remoting.Clients.*
            end
            obj.clientManager = ClientManager.Create();
            obj.client = obj.clientManager.CreateRequestClient(IPCport);
            

            obj.chip_cont=['\system\Subsystem_' num2str(subsys) '\AD9082-FMCA-EBZ\AD9082'];
            obj.brd_cont=['\system\Subsystem_' num2str(subsys) '\AD9082-FMCA-EBZ'];
            
            obj.ads9_cont=['\system\Subsystem_' num2str(subsys) '\ADS9-V2'];
            %obj.ads9_cont=['\system\Subsystem_' num2str(subsys) '\ADS9-V2\AD9082 ADS9 FPGA'];
            
            
            
            obj.client.ConnectionTimeout= 40000;
            obj.client.SendTimeout= 40000;
            obj.client.ReceiveTimeout= 40000;
            
            
            if(nargin>2)
                obj.fdac=varargin{1};
                
            else
                
                
                obj.fdac=12e9;
            end
            
            if(nargin>3)
                obj.fadc=varargin{2};
                
            else
                
                
                obj.fadc=4e9;
            end
            
            
        end
        
        %%%ch_ix --> ADC channel, use 0 for AD9081
        
        function datcap=captureAllChannels(obj)
            obj.client.ContextPath=obj.chip_cont;
            %obj.client.set_ContextPath(obj.chip_cont);  --> may be needed
            %for labVIEW
            ranix=floor(abs(1e6*randn(1)));
            fname=['\tmpFile_' num2str(ranix)];
            delete([pwd '\tmpFile*']);
            
            obj.client.Run('@AsyncDataCapture(tagx)');
            %%%% Increase the pause duration for longer captures.
            
            disp('Capturing Waveform ... ')
            pause(5);
            disp('Waveform Captured')
            
            
            obj.client.PullAllCaptureDataToStore('-1','2000','True','True','tagx','','');
            %obj.client.PullAllCaptureDataToStore('-1','2000','False','True','tagx','','None');
            
            
            pause(0.2);
            
            try
                obj.client.ExportStoreDataToFile([pwd fname],'csv','','rawsamples');
                datin=importdata([pwd fname '_rawsamples.csv'],',');
                datcap=datin.data;
                
            catch
                disp('file capture error');
                datcap=[];
                
            end
            
            
            
            
        end
        
        
        
        function setComplex(obj,varargin)
            obj.client.ContextPath=obj.chip_cont;
            %obj.client.set_ContextPath(obj.chip_cont);  --> may be needed
            %for labVIEW
            
            obj.client.RawWriteRegister('0x1d','0x4');
            
            obj.client.RawWriteRegister('0x1b','0x1');
            obj.client.RawWriteRegister('0x1c9','0x78');
            obj.client.RawWriteRegister('0x1b','0x2');
            obj.client.RawWriteRegister('0x1c9','0x70');
            if(nargin>1)
                if(varargin{1})
                    obj.client.RawWriteRegister('0x280','0x1');
                end
                
            end
            
        end
        
        
        
        
        
        function setFSC(obj,gain_enable)
            obj.client.ContextPath=obj.chip_cont;
            %obj.client.set_ContextPath(obj.chip_cont);  --> may be needed
            %for labVIEW
            
            obj.client.RawWriteRegister('0x1B','0x0F');
            
            
            if(gain_enable)
                obj.client.RawWriteRegister('0x117', '0xA0');
                obj.client.RawWriteRegister('0x118', '0xFF');
                
                
            else
                obj.client.RawWriteRegister('0x117', '0x40');
                obj.client.RawWriteRegister('0x118', '0x88');
                
            end
            
            
            
            
        end
        
        function setFSCval(obj,fscval)
            obj.client.ContextPath=obj.chip_cont;
            %obj.client.set_ContextPath(obj.chip_cont);  --> may be needed
            %for labVIEW
            
            obj.client.RawWriteRegister('0x1B','0x0F');
            fsc0=10*2^4;
            
            if(fscval>40.5)
                fscval=40.5;
            elseif(fscval<15.6)
                fscval=15.6;
            end
            
            icode=fix((1024/25)*(fscval-15.6));
            fsc0=10*2^4+bitand(icode,3);
            fsc1=bitshift(icode,-2);
            
            
            
            
            obj.client.RawWriteRegister('0x117', ['0x' dec2hex(fsc0)]);
            obj.client.RawWriteRegister('0x118', ['0x' dec2hex(fsc1)]);
            
            
            
        end
        
        %Freq  - NCO frequency
        %mask  - Which NCOs to set their frequency.
        %        '11111111' will write to all channels gains
        %        '10101010' will write to the gains of ch0, ch2, ch4 and ch6
        %        '01010101' will write to the gains of ch1, ch3, ch5 and ch7
        function setTXfineGain(self, gain, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(8) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_7', num2str(gain), '-1');
            end 
            if mask(7) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_6', num2str(gain), '-1');
            end
            if mask(6) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_5', num2str(gain), '-1');
            end
            if mask(5) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_4', num2str(gain), '-1');
            end 
            if mask(4) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_3', num2str(gain), '-1');
            end 
            if mask(3) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_2', num2str(gain), '-1');
            end
            if mask(2) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_1', num2str(gain), '-1');
            end
            if mask(1) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_ch_gain_0', num2str(gain), '-1');
            end 
            
            self.client.SetBoolParameter('virtual-parameter-txgain_exec_seq', 'True', '-1');
            disp("Wait for setting TX Channels Gain...");
            pause(3)
            self.client.SetBoolParameter('virtual-parameter-txgain_exec_seq', 'False', '-1');
            disp("Done");

        end
        
        %Freq  - TxNCO frequency
        %mask  - Which NCOs to set their frequency.
        %        '1111' will write to all ncos
        %        '1010' will write to NCO1 and NCO3
        %        '0101' will write to NCO0 and NCO2
        function ADCsetNCOfreq(self, freq, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(4) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_coarse_nco_0', num2str(freq), '-1');
            end 
            if mask(3) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_coarse_nco_1', num2str(freq), '-1');
            end
            if mask(2) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_coarse_nco_2', num2str(freq), '-1');
            end
            if mask(1) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_coarse_nco_3', num2str(freq), '-1');
            end 
            
            self.client.SetBoolParameter('virtual-parameter-rxcoarsenco_exec_seq', 'True', '-1');
            self.client.SetBoolParameter('virtual-parameter-rxcoarsenco_exec_seq', 'False', '-1');
        end
        
         %Freq  - TxNCO frequency
        %mask  - Which NCOs to set their frequency.
        %        '1111' will write to all ncos
        %        '1010' will write to NCO1 and NCO3
        %        '0101' will write to NCO0 and NCO2
        function DACsetNCOfreq(self, freq, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(4) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_coarse_nco_0', num2str(freq), '-1');
            end 
            if mask(3) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_coarse_nco_1', num2str(freq), '-1');
            end
            if mask(2) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_coarse_nco_2', num2str(freq), '-1');
            end
            if mask(1) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_coarse_nco_3', num2str(freq), '-1');
            end 
            
            self.client.SetBoolParameter('virtual-parameter-txcoarsenco_exec_seq', 'True', '-1');
            self.client.SetBoolParameter('virtual-parameter-txcoarsenco_exec_seq', 'False', '-1');
        end
        
        %### dmask is calculated as follows:
        %### 1 for DAC0
        %### 2 for DAC1
        %### 4 for DAC2
        %### 8 for DAC3
        %### 1+4 for DAC0 & DAC2
        %### 2+8 for DAC1 & DAC3
        %### 1 + 2 + 8 for DAC0, DAC1, and DAC3
        
        function setdacfreq(self,freq,dmask)
            self.client.ContextPath=self.chip_cont;
            
            self.client.RawWriteRegister('0x1b',hex(dmask));
            
            
            ftw=(floor( (2^48) * (freq)/(self.fdac) ));
            
            
            ftw0=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            ftw1=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw2=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw3=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw4=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw5=bitand(ftw,255);
            
            
            self.client.RawWriteRegister('0x01ca',hex(0));
            self.client.RawWriteRegister('0x01cb',hex(ftw0));
            self.client.RawWriteRegister('0x01cc',hex(ftw1));
            self.client.RawWriteRegister('0x01cd',hex(ftw2));
            self.client.RawWriteRegister('0x01ce',hex(ftw3));
            self.client.RawWriteRegister('0x01cf',hex(ftw4));
            self.client.RawWriteRegister('0x01d0',hex(ftw5));
            self.client.RawWriteRegister('0x01ca',hex(0));
            self.client.RawWriteRegister('0x01ca',hex(1));
            %self.client.RawWriteRegister('0x01ca',hex(0));
            
        end
        
        
        
        %### dmask = 0xff = 255
        
        
        function setadcfreq(self,freq,dmask)
            self.client.ContextPath=self.chip_cont;
            
            self.client.RawWriteRegister('0x18',hex(dmask));
            
            
            ftw=(floor( (2^48) * (freq)/(self.fadc) ));
            
            
            ftw0=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            ftw1=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw2=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw3=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw4=bitand(ftw,255);
            ftw=bitshift(ftw,-8);
            
            
            ftw5=bitand(ftw,255);
            hex(ftw5);
            
            %self.client.RawWriteRegister('0xa1f',hex(0));
            self.client.RawWriteRegister('0xa05',hex(ftw0));
            self.client.RawWriteRegister('0xa06',hex(ftw1));
            self.client.RawWriteRegister('0xa07',hex(ftw2));
            self.client.RawWriteRegister('0xa08',hex(ftw3));
            self.client.RawWriteRegister('0xa09',hex(ftw4));
            self.client.RawWriteRegister('0xa0a',hex(ftw5));
            % self.client.RawWriteRegister('0xa1f',hex(0));
            %self.client.RawWriteRegister('0x01ca',hex(0));
            
        end
        
        
        
        
        %### cmask is calculated as follows:
        %### 1 for DAC0
        %### 2 for DAC1
        %### 4 for DAC2
        %### 8 for DAC3
        %### 1+4 for DAC0 & DAC2
        %### 2+8 for DAC1 & DAC3
        %### 1 + 2 + 8 for DAC0, DAC1, and DAC3
        function setDClev(obj,dcval,cmask)
            obj.client.ContextPath=obj.chip_cont;
            
            
            obj.client.RawWriteRegister('0x1C',hex(cmask));
            dccalc=floor(floor(23170*(10^(double(dcval)/20))));
            
            
            
            
            dccalc_0=bitand(dccalc , 255);
            dccalc=bitshift(dccalc,-8);
            dccalc_1=bitand(dccalc , 255);
            
            
            
            obj.client.RawWriteRegister('0x1b7',hex(dccalc_1));
            obj.client.RawWriteRegister('0x1b6',hex(dccalc_0));
            
        end
        
        %8 digit mask
        function ADCsetFineNCOfreq(self, freq, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(8) == '1'
            self.client.SetDecimalParameter("virtual-parameter-rx_fine_nco_0", num2str(freq), '-1');
            end 
            if mask(7) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_fine_nco_1', num2str(freq), '-1');
            end
            if mask(6) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_fine_nco_2', num2str(freq), '-1');
            end
            if mask(5) == '1'
            self.client.SetDecimalParameter('virtual-parameter-rx_fine_nco_3', num2str(freq), '-1');
            end
            if mask(4) == '1'
            self.client.SetDecimalParameter("virtual-parameter-rx_fine_nco_4", num2str(freq), '-1');
            end
            if mask(3) == '1'
            self.client.SetDecimalParameter("virtual-parameter-rx_fine_nco_5", num2str(freq), '-1');
            end
            if mask(2) == '1'
            self.client.SetDecimalParameter("virtual-parameter-rx_fine_nco_6", num2str(freq), '-1');
            end
            if mask(1) == '1'
            self.client.SetDecimalParameter("virtual-parameter-rx_fine_nco_7", num2str(freq), '-1');
            end

            disp("ADCsetFineNCOfreq Done");

        end

        %8 digit mask
        function DACsetFineNCOfreq(self, freq, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(8) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_fine_nco_0", num2str(freq), '-1');
            end 
            if mask(7) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_fine_nco_1', num2str(freq), '-1');
            end
            if mask(6) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_fine_nco_2', num2str(freq), '-1');
            end
            if mask(5) == '1'
            self.client.SetDecimalParameter('virtual-parameter-tx_fine_nco_3', num2str(freq), '-1');
            end
            if mask(4) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_fine_nco_4", num2str(freq), '-1');
            end
            if mask(3) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_fine_nco_5", num2str(freq), '-1');
            end
            if mask(2) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_fine_nco_6", num2str(freq), '-1');
            end
            if mask(1) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_fine_nco_7", num2str(freq), '-1');
            end

            disp("DACsetFineNCOfreq Done");

        end

        %8 digit mask
        function DACsetDCOffsetGain(self, gain, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(8) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_0", num2str(gain), '-1');
            end 
            if mask(7) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_1", num2str(gain), '-1');
            end
            if mask(6) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_2", num2str(gain), '-1');
            end
            if mask(5) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_3", num2str(gain), '-1');
            end 
            if mask(4) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_4", num2str(gain), '-1');
            end 
            if mask(3) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_5", num2str(gain), '-1');
            end
            if mask(2) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_6", num2str(gain), '-1');
            end
            if mask(1) == '1'
            self.client.SetDecimalParameter("virtual-parameter-tx_dc_offset_7", num2str(gain), '-1');
            end
           
            self.client.SetBoolParameter('virtual-parameter-txgain_exec_seq', 'True', '-1');
            pause(10)
            self.client.SetBoolParameter('virtual-parameter-txgain_exec_seq', 'False', '-1');
            disp("DACsetDCOffsetGain Done");

        end

        %ToF - 0 for False/Off, 1 for True/On
        %4 digit mask
        function ADCCoarseDDCGain(self, ToF, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(4) == '1'
            self.client.SetIntParameter("virtual-parameter-cddc_0_3db_gain", num2str(ToF), '-1');
            end
            if mask(3) == '1'
            self.client.SetIntParameter("virtual-parameter-cddc_1_3db_gain", num2str(ToF), '-1');
            end
            if mask(2) == '1'
            self.client.SetIntParameter("virtual-parameter-cddc_2_3db_gain", num2str(ToF), '-1');
            end
            if mask(1) == '1'
            self.client.SetIntParameter("virtual-parameter-cddc_3_3db_gain", num2str(ToF), '-1');
            end

            self.client.SetBoolParameter("virtual-parameter-rxgain_exec_seq", 'True', '-1');
            self.client.SetBoolParameter("virtual-parameter-rxgain_exec_seq", 'False', '-1');

            disp("ADCCoarseDDCGain Done");

        end

        %8 digit mask
        function ADCFineDDCGain(self, ToF, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(8) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_0_3db_gain", num2str(ToF), '-1');
            end
            if mask(7) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_1_3db_gain", num2str(ToF), '-1');
            end
            if mask(6) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_2_3db_gain", num2str(ToF), '-1');
            end
            if mask(5) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_3_3db_gain", num2str(ToF), '-1');
            end
            if mask(4) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_4_3db_gain", num2str(ToF), '-1');
            end
            if mask(3) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_5_3db_gain", num2str(ToF), '-1');
            end
            if mask(2) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_6_3db_gain", num2str(ToF), '-1');
            end
            if mask(1) == '1'
            self.client.SetIntParameter("virtual-parameter-fddc_7_3db_gain", num2str(ToF), '-1');
            end

            self.client.SetBoolParameter("virtual-parameter-rxgain_exec_seq", 'True', '-1');
            self.client.SetBoolParameter("virtual-parameter-rxgain_exec_seq", 'False', '-1');

            disp("ADCFineDDCGain Done");

        end

        %4 digit mask
        function DACFSCSetting(self, mamp, mask)
            self.client.ContextPath=self.chip_cont;
            if mask(4) == '1'
            self.client.SetIntParameter("virtual-parameter-tx_fsc_0", num2str(mamp), '-1');
            end
            if mask(3) == '1'
            self.client.SetIntParameter("virtual-parameter-tx_fsc_1", num2str(mamp), '-1');
            end
            if mask(2) == '1'
            self.client.SetIntParameter("virtual-parameter-tx_fsc_2", num2str(mamp), '-1');
            end
            if mask(1) == '1'
            self.client.SetIntParameter("virtual-parameter-tx_fsc_3", num2str(mamp), '-1');
            end

            self.client.SetBoolParameter("virtual-parameter-rxgain_exec_seq", 'True', '-1');
            self.client.SetBoolParameter("virtual-parameter-rxgain_exec_seq", 'False', '-1');

            disp("DACFSCSetting Done");

        end
        
        %Count has to be a number -> 2^#
        function SampleCount(self, count)
            self.client.ContextPath=self.chip_cont;
            self.client.SetBigIntParameter("virtual-parameter-validatedSampleCount", num2str(count), '-1');

            disp("SampleCount Done");

        end

        %You may change the path to the one you desire, keep the
        %'.acesamples' at the end of your desired path
        function ExportFile(obj)
            obj.client.ContextPath=obj.chip_cont;
            obj.client.ExportStoreDataToFile("C:\Users\***\AppData\Local\Analog Devices\ACE\ExportData\AD9082\.acesamples", "acesamples", "", "");

            disp("ExportFile Done");

        end


    end
end