function [Idata, Qdata, actual]  = CreateTone(freq, numSamples, fdata, dBfs, resolution)
%
%[Idata, Qdata, actual]  = CreateTone(freq, numSamples, fdata, dBfs, resolution)
% 
% Create a simple tone
%
% freq       - desired tone freq
% numSamples - number of samples in waveform
% fdata      - sample data rate
% dBfs       - back off from full scale
% resolution - DAC resolution. depends on the JESD mode. 
% actualFreq - Actual generated frequency. different from requested to get
%              contigous phase. 
%
disp(['Creating a tone at: ', num2str(freq/1e6), 'MHz numSamples: ', num2str(numSamples)]);

% Signed Example
desired = freq;       % Output frequency in Hz
samples = numSamples;
cycles = round(desired*samples/fdata);

if (mod(cycles, 2) == 0)
    %disp('oddifying');
    cycles = cycles + 1;
end

disp(['Creating tone with ', num2str(samples), ' samples']);

% Calculating Frequency to be created
actual=cycles*fdata/samples;
disp(['Actual Frequency ', num2str(actual/1e6), ' MHz']);
disp(newline);

% Creating I and Q data
x=[0:(2*cycles*pi/samples):2*cycles*pi-(2*cycles*pi/samples)];
Idata = round(10^(dBfs/20)*(2^(resolution-1)-1)*cos(x));  % I data is a cosine wave
Qdata = round(10^(dBfs/20)*(2^(resolution-1)-1)*sin(x));  % Q data is a sine wave


end