function [Idata, Qdata, actual1, actual2]  = CreateTwoTone(freq1, freq2, numSamples, fdata, dBfs, resolution)
%
%[Idata, Qdata, actual1, actual2]  = CreateTwoTone(freq1, freq2, numSamples, fdata, dBfs, resolution)
% 
%Create two tones
%
% freq1       -first desired tone freq
% freq2       -second desired tone freq
% numSamples - number of samples in waveform
% fdata      - sample data rate
% dBfs       - back off from full scale
% resolution - DAC resolution. depends on the JESD mode. 
% actual1    - Actual first tone freq
% actual2    - Actual second tone freq

disp(['Creating Two tones at: ', num2str(freq1/1e6),'MHz, and at ',num2str(freq2/1e6), 'MHz numSamples: ', num2str(numSamples)]);

% Signed Example
desired1 = freq1;       % Output frequency in Hz
desired2 = freq2;       % Output frequency in Hz

samples = numSamples;
cycles1 = round(desired1*samples/fdata);
cycles2 = round(desired2*samples/fdata);

if (mod(cycles1, 2) == 0)
    %disp('oddifying');
    cycles1 = cycles1 + 1;
end


if (mod(cycles2, 2) == 0)
    %disp('oddifying');
    cycles2 = cycles2 + 1;
end

disp(['Creating tone with ', num2str(samples), ' samples']);

actual1=cycles1*fdata/samples;
actual2=cycles2*fdata/samples;
disp(['Actual Frequency of first tone ', num2str(actual1/1e6), ' MHz']);
disp(['Actual Frequency of second tone ', num2str(actual2/1e6), ' MHz']);

disp(newline);

x1=[0:(2*cycles1*pi/samples):2*cycles1*pi-(2*cycles1*pi/samples)];
x2=[0:(2*cycles2*pi/samples):2*cycles2*pi-(2*cycles2*pi/samples)];

Idata = round(10^(dBfs/20)*(2^(resolution-1)-1)*(1*cos(x1)+1*cos(x2)));  % I data is a cosine wave
Qdata = round(10^(dBfs/20)*(2^(resolution-1)-1)*(1*sin(x1)+1*sin(x2)));  % Q data is a sine wave
end