%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EXAMPLE MXFE AUTOMATION:
%
%  THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
%  INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
%  PARTICULAR PURPOSE ARE DISCLAIMED.
%
%  IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
%  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
%  RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
%  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
%  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
%  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

classdef NDPG9xxx
    properties
        
        asm
        ads9
        ad9986
        jesd204_params0
        jesd204_params1
        dpg
        
    end
    methods
        function self = NDPG9xxx()
            PluginDir='C:\ProgramData\Analog Devices\ACE\Plugins';
            ACEDir='C:\Program Files (x86)\Analog Devices\ACE';
            
            try
                NET.addAssembly([PluginDir '\Hardware.SdpSupport.1.2022.22100\lib\sdpApi1.dll']);
                NET.addAssembly([ACEDir '\AnalogDevices.Csa.dll']);
                NET.addAssembly([ACEDir '\AnalogDevices.Csa.UI.dll']);
                NET.addAssembly([PluginDir '\Support.AnalogDevices.DPG.HardwareControl.1.19.187\lib\net452\AnalogDevices.DPG.HardwareControl.dll']);
                NET.addAssembly([ACEDir '\AnalogDevices.Csa.Hardware.dll']);
                NET.addAssembly([PluginDir '\Support.AnalogDevices.Vector.Common.1.19.187\lib\net452\AnalogDevices.Vector.Common.dll']);
                NET.addAssembly([PluginDir '\Support.AnalogDevices.DPG.Interfaces.1.19.187\lib\net452\AnalogDevices.DPG.Interfaces.dll']);
                NET.addAssembly([PluginDir '\Support.AnalogDevices.DPG.EvalBoard.AD9986.1.19.187\lib\net452\AnalogDevices.DPG.EvalBoard.AD9986.dll']);
                NET.addAssembly([PluginDir '\Support.AnalogDevices.DPG.Hardware.ADS9.1.19.187\lib\net452\AnalogDevices.DPG.Hardware.ADS9.dll']);
                NET.addAssembly([PluginDir '\Support.AnalogDevices.DPG.Hardware.ADS9.1.19.187\lib\net452\AnalogDevices.DPG.Hardware.ADS9.dll']);

            end
            %get ADS9 handle
            self.ads9 = self.GetAds9();   % true = force FPGA config download, false = auto
            self.dpg=self.ads9;
            self.ad9986 = AnalogDevices.DPG.EvalBoard.AD9986.RemoteEvalApi(self.ads9);
            self.jesd204_params0 = self.ad9986.LinkParams0;
            self.jesd204_params1 = self.ad9986.LinkParams1;
        end
        
        %Play Vector
        function play(self)
            self.ad9986.TransmitStart()
            self.ad9986.Play2(true)
        end
        
        %Stop Vector
        function stop(self)
            self.ad9986.Stop()
            self.ad9986.TransmitStop()
        end
        
        %Check link and replay the vector
        function replay(obj)
            linkRcount=0;
            while(~obj.get_sync_state && linkRcount<10)
                linkRcount=linkRcount+1;
                obj.play();
            end
        end
        
        %FPGA register read access
        function val=read_fpga_register(self, reg)
            val = self.ad9986.Ad9986Jesd204Tx.FpgaReadRegister_(reg)
        end
        %FPGA register Write access
        function write_fpga_register(self, reg, val)
            self.ad9986.Ad9986Jesd204Tx.FpgaWriteRegister_(reg, val);
        end
        
        %Check sync status
        function [stat0]=get_sync_state(self)
            retval= string(self.ad9986.SyncState0);
            stat0= strcmpi(retval,'High');
            
            %stat1= self.ad9986.SyncState1;
        end
        
        %check the link status
        function [stat0]=checkLink(self)
            retval= string(self.ad9986.SyncState0);
            stat0= strcmpi(retval,'High');
            
            %stat1= self.ad9986.SyncState1;
        end
        
        
        %Check if the link is up
        function [success]=linkup(self)
            numChks=40;
            for i=1:numChks
                linkStat(i)=self.get_sync_state();
                pause(0.01);
            end
            success = (sum(linkStat) == numChks);
        end
        

        function dowloadWFM(self,dpgArray,autosize)
        %
        % Complex WFM download
        % dpgArray --> 2xN waveform, 1,N -->I, 2,N -->Q. Integer values
        % -2^(res-1) to +2^(res-1)-1
        % autosize --> the array needs to be MxN where M is number of converters
        % based on the JESD config. If the array is not MxN, then autosize repeats
        % the array based M/X and downloads the same vector on all DACs
        %
            self.write_fpga_register(hex2dec('533'),hex2dec('80000000'));
            self.write_fpga_register(hex2dec('540'),hex2dec('3'));
            
            if (autosize)
                rpt_cnt=uint8(self.jesd204_params0.M)/size(dpgArray,1);
                dpgArrayx=dpgArray;
                for i=1:rpt_cnt-1
                    dpgArrayx=[dpgArrayx;dpgArray];
                end
                dpgArray=dpgArrayx;
                % size(dpgArray);
            end
            self.ad9986.DownloadVectors(0, dpgArray);
            pause(1);
            self.play();
        end
        
        %get ADS9 handle
        function ads9=GetAds9(self)
            dpg_interface = AnalogDevices.DPG.Hardware.ADS9.DPG();
            devices = dpg_interface.AttachedDevices;
            devList=devices;
            if(devList.Length>1)
                for i=1:devList.Length
                    device_name=devList(i).FriendlyName;
                    
                    if(strcmp(char(device_name),'ADS9-V2'))
                        dev_index=i;
                    end
                end
            else
                dev_index=1;
            end
            ads9 = devices(dev_index);
        end
    end
end


